/***
  BEGIN LICENSE
	
  Copyright (C) 2011 Mario Guerriero <mefrio.g@gmail.com>	
  This program is free software: you can redistribute it and/or modify it	
  under the terms of the GNU Lesser General Public License version 3, as published	
  by the Free Software Foundation.
	
  This program is distributed in the hope that it will be useful, but	
  WITHOUT ANY WARRANTY; without even the implied warranties of	
  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR	
  PURPOSE.  See the GNU General Public License for more details.
	
  You should have received a copy of the GNU General Public License along	
  with this program.  If not, see <http://www.gnu.org/licenses/>	
  
  END LICENSE	
***/

using Gtk;

public class PlankPlug : Pantheon.Switchboard.Plug {

    public PlankPlug () {
		
        var plug = new Plank ();
        this.add(plug);
    
    }
}


public class Settings {
    public int iconsize { get; set; default = 48; }
    public int hidemode { get; set; default = 1; }
    public int monitor { get; set; default = 0; }
}

enum HideType {
    NEVER,
    INTELLIGENT,
    ALWAYS
}

public class Plank : VBox {
    string plank_config_file;
    string config_dir;
    GLib.KeyFile plank_settings;
    Settings settings = new Settings ();
    string key_group = "PlankDockPreferences";

    ComboBoxText hidemode;
    ComboBoxText monitor;
    HScale iconsize;
    Label pix_label;

    public Plank () {
        config_dir = Environment.get_user_config_dir ();
        plank_config_file = config_dir + "/plank/dock1/settings";

        load ();
		
        var label = new Label ("<b>" + _("Icon size") + ": </b>");
        label.use_markup = true;
        pix_label = new Label ("px");
        set_pixel_label ();
        iconsize = new HScale.with_range (24.0, 128.0, 1.0);
        iconsize.set_draw_value (false);
        iconsize.set_value (settings.iconsize);
        iconsize.value_changed.connect (apply);
        var hbox = new HBox (false, 4);
		hbox.pack_start (label, false, false, 25);
		hbox.pack_end (pix_label, false, false, 25);
        pack_start (hbox, false, false, 10);
        hbox = new HBox (false, 4);
		hbox.pack_start (iconsize, true, true, 45);
        pack_start (hbox, false, true, 0);
        
        label = new Label ("<b>" + _("Hide mode") + ": </b>");
        label.use_markup = true;
        hidemode = new ComboBoxText ();
        hidemode.append_text (_("Never"));
        hidemode.append_text (_("Intelligent"));
        hidemode.append_text (_("Always"));
        hidemode.set_active (settings.hidemode);
        hidemode.changed.connect (apply);
        hbox = new HBox (false, 4);
		hbox.pack_start (label, false, false, 25);
        pack_start (hbox, false, false, 10);
        hbox = new HBox (false, 4);
		hbox.pack_start (hidemode, false, false, 45);
        pack_start (hbox, false, true, 0);
        
        label = new Label ("<b>" + _("Monitor") + ": </b>");
        label.use_markup = true;
        monitor = new ComboBoxText ();
		monitor.append_text ("0");
		monitor.append_text ("1");
		monitor.append_text ("2");
		monitor.set_active (settings.monitor);
		monitor.changed.connect (apply);
        hbox = new HBox (false, 4);
		hbox.pack_start (label, false, false, 25);
        pack_start (hbox, false, false, 10);
        hbox = new HBox (false, 4);
		hbox.pack_start (monitor, false, false, 45);
        pack_start (hbox, false, true, 0);

		stdout.printf ("\n\n\n\n");
        show_all ();
    }

    void apply () {
        settings.iconsize = (int) iconsize.get_value ();
        settings.hidemode = hidemode.get_active ();
        settings.monitor = monitor.get_active ();

        plank_settings.set_integer (key_group, "IconSize", settings.iconsize);
        plank_settings.set_integer (key_group, "HideMode", settings.hidemode);
        plank_settings.set_integer (key_group, "Monitor", settings.monitor);
		
		set_pixel_label ();
		
        try {
            write_plank_settings ();
        } catch (GLib.Error error) {
            GLib.critical ("Failed to save \"%s\": %s", plank_config_file, error.message);
        }

    }

    void write_plank_settings () throws Error {
        string config_path = Environment.get_user_config_dir () + "/plank/dock1";
        if (DirUtils.create_with_parents (config_path, 0700) != 0)
            throw new GLib.FileError.FAILED ("Config folder couldn't be created.");
        File f = File.new_for_path(plank_config_file);
        if (f.query_exists(null)) {
            f.delete(null);
        }
        var f_stream = f.create(FileCreateFlags.REPLACE_DESTINATION, null);
        var d_stream = new DataOutputStream(f_stream);
        d_stream.put_string(plank_settings.to_data());
        
        set_pixel_label ();
    }

    void load () {
        /* Load plank settings */
        plank_settings = new GLib.KeyFile ();
        try {
            plank_settings.load_from_file (plank_config_file, 0);
        } catch (GLib.Error error) {
            GLib.debug ("Failed to load \"%s\": %s", plank_config_file, error.message);
        }

        try {
            if (plank_settings.has_key (key_group, "IconSize"))
                settings.iconsize = plank_settings.get_integer (key_group, "IconSize");
            if (plank_settings.has_key (key_group, "HideMode"))
                settings.hidemode = plank_settings.get_integer (key_group, "HideMode");
            if (plank_settings.has_key (key_group, "Monitor"))
                settings.monitor = plank_settings.get_integer (key_group, "Monitor");
        } catch (GLib.KeyFileError error) {
            GLib.debug ("Failed to load some preferences from \"%s\": %s", plank_config_file, error.message);
        }
        
        set_pixel_label ();
    }
    
    void set_pixel_label () {
		pix_label.set_text (settings.iconsize.to_string () + " px");
	}

}

public static int main (string[] args) {

    Gtk.init (ref args);
    var plug = new PlankPlug ();
    plug.register ("Plank");
    plug.show_all ();
    Gtk.main ();
    return 0;
}
